#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import unicodedata

def removeControlChar(string):
	"""
	제어문자를 제거합니다.
	
	:param string: 문자열
	:type string: unicode
	
	:return: 경로 문자가 제거된 문자열
	:rtype: unicode
	"""
	return ''.join(ch for ch in string if unicodedata.category(ch)[0]!="C")



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess
import pprint
import os

class Notepad(object):
	def open(self):
		"""
		메모창을 엽니다.
		"""
		self._devnull = open(os.devnull, 'w')
		self._process = subprocess.Popen(['kate', '--stdin'], stdin=subprocess.PIPE, stdout=self._devnull, stderr=self._devnull)
		pass
	
	def write(self, content):
		"""
		메모에 내용을 씁니다.
		
		:param content: 쓸 내용. list의 경우 각 항목을 줄 단위로 분할하여 기록합니다. list내에 존재하는 unicode는 그대로 기록하고, 그 외는 pprint.pformat함수를 호출하여 텍스트로 바꾸어 기록합니다. 그 외 요소는 모두 pprint.pformat함수를 호출하여 기록합니다.
		"""
		if isinstance(content, unicode):
			write = content.encode('UTF-8')
		elif isinstance(content, list):
			write = list()
			for i in content:
				if isinstance(i, unicode):
					write.append(i.encode('UTF-8'))
				else:
					write.append(pprint.pformat(i))
			write = '\n'.join(write)
		else:
			write = pprint.pformat(content)
		self._process.stdin.write(write)
		self._process.stdin.close()
	
	def close(self):
		"""
		메모창을 닫습니다.
		"""
		self._process.terminate()
		self._devnull.close()
		pass
	
	@classmethod
	def show(cls, content):
		"""
		주어진 내용을 보여줍니다.
		
		:param content: 보여줄 내용. :py:func:`~ufp.gui.Notepad.write` 의 내용과 같습니다.
		:return: 객체를 반환합니다.
		:rtype: :py:class:`~ufp.gui.Notepad`
		"""
		notepad = cls()
		notepad.open()
		notepad.write(content)
		return notepad
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import random
import string as _p_string

def string(length, chars=_p_string.ascii_letters + _p_string.digits):
	"""
	지정된 길이 만큼의 램덤 문자열을 되돌려줍니다.
	
	:param length: 문자열의 길이
	:type length: int
	:param chars: 문자 집합\n
		[a, ..., z, A, ..., Z, 0, ..., 9] (기본값)
	:type chars: str, unicode, list
	:return: 램덤 문자열
	:rtype: unicode
	"""
	string = unicode()
	for i in xrange(length):
		string += random.choice(chars)
		pass
	return string
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess
import struct
import time
import os

_ghostscriptCommand = [
	'gs',
	'-q',
	'-dQUIET',
	'-dSAFER',
	'-dBATCH',
	'-dNOPAUSE',
	'-dNOPROMPT',
	'-dMaxBitmap=500000000',
	'-dAlignToPixels=0',
	'-dGridFitTT=2',
	'-dUseCropBox', # PDF 페이지 박스 내의 내용만 저장
	'-dTextAlphaBits=4',
	'-dGraphicsAlphaBits=4',
	'-sOutputFile=%stdout', #출력을 표준출력으로
	'-' #입력을 표준입력에서
	]

def toBmps(pdf, format='bmp16m', dpi=200):
	"""
	pdf 파일을 bmp로 바꾸어 저장합니다.
	
	각 페이지의 bmp는 제너레이터 형식으로 각각 반환됩니다. 예를 들어 다음과 같이 작동합니다.
	
	.. code-block:: python
	
		import ufp.pdf
		
		with open('test.pdf', 'r') as f:
			pdf = f.read()
		
		buffer = ufp.pdf.toBmps(pdf, format='bmp16m')
		for page, bmp in enumerate(buffer, start=1):
			with open('{0}.bmp'.format(page), 'w') as f:
				f.write(bmp)
	
	..
	   비트맵 파일 구조 : http://yagi815.tistory.com/242
	   enumerate 내장 함수 설명 : https://docs.python.org/2/library/functions.html#enumerate
	
	:param pdf: pdf 바이너리 데이터.
		여러 페이지가 포함된 파일도 허용합니다.
	:type pdf: bytes
	:param format: 출력 포멧.\n
		bmp16m: 24-bit RGB Color\n
		bmpmono: Black-and-White Color\n
		bmpgray: Grayscale Color\n
		bmpsep1:\n
		bmpsep8:\n
		bmp16: 4-bit Color\n
		bmp256: 8-bit Color\n
		bmp32b: 32-bit RGBA Color
	:type format: unicode
	:param dpi: DPI.
		기본값은 200DPI입니다.
		양의 정수만을 취합니다.
	:type dpi: int
	:raise Exception: pdf 변환에 문제가 발생했을때.
	:yield: BMP 이미지 바이너리 데이터(bytes)
	"""
	#명령 설정
	cmd = _ghostscriptCommand[:]
	cmd.insert(-2, '-sDEVICE={format}'.format(format=format)) #출력 형식
	cmd.insert(-2, '-r{dpi}x{dpi}'.format(dpi=dpi))  #DPI
	
	try:
		#실행
		devnull = open(os.devnull, 'w')
		gs = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=devnull)
		
		#pdf 파일 입력
		gs.stdin.write(pdf)
		gs.stdin.close()
		
		preBytes = None
		while True:
			#만약 출력 버퍼가 닫겼다면 종료.
			if gs.poll() is not None:
				if gs.returncode != 0:
					raise Exception('pdf -> bmp 변환에 문제가 있습니다.')
				raise StopIteration
			
			# bfType, bfSize 읽어오기
			if preBytes:
				bytes = preBytes + gs.stdout.read(6 - len(preBytes))
				preBytes = None
			else:
				bytes = gs.stdout.read(6)
			
			#못 읽어왔으면 기다렸다 다시 시도
			if len(bytes) != 6:
				time.sleep(0.1)
				preBytes = bytes
				continue
			
			# bfType, bfSize 파싱
			bfType, bfSize = struct.unpack(b'<2sI', bytes)
			
			if bfType != 'BM':
				raise Exception('pdf -> bmp 변환에 문제가 있습니다.')
			
			#bmp 파일 작성
			bmp = bytes
			bmp += gs.stdout.read(bfSize-6)
			
			#값을 반환
			yield bmp
	except GeneratorExit:
		if gs.poll() is not None:
			gs.terminate()
		devnull.close()
	pass

def toBmp(pdf, format='bmp32b', dpi=200):
	"""
	pdf 파일을 bmp로 바꾸어 저장합니다.
	
	다음과 같은 식으로 사용합니다.
	
	.. code-block:: python
	
		import pyPdf
		from io import BytesIO
		import ufp.pdf
		
		#PDF 파일을 준비
		buffer = file(srcPath, 'rb')
		pdf = pyPdf.PdfFileReader(buffer)
		
		#각 페이지 마다
		for pageNumber in xrange(pdf.getNumPages()):
			#현재 페이지의 pdf 데이터를 추출
			page = pdf.getPage(pageNumber)
			pdfFileWriter = pyPdf.PdfFileWriter()
			pdfFileWriter.addPage(page)
			with BytesIO() as outputStream:
				pdfFileWriter.write(outputStream)
				pageBinary = outputStream.getvalue()
				pass
			
			#PDF를 BMP(24-bit RGB Color)로 변환
			imageBinary = ufp.pdf.toBmp(pageBinary, format='bmp16m', dpi=200)
			
			#저장
			with open('{0}.bmp'.format(pageNumber), 'w') as f:
				f.write(imageBinary)
			pass
	
	.. 고스트 스크립트 문서 : http://www.ghostscript.com/doc/current/Readme.htm
	
	:param pdf: pdf 바이너리 데이터.
		오직 1 페이지만 존재하는 pdf 데이터여야 합니다.
	:type pdf: bytes
	:param format: 출력 포멧.\n
		bmp16m: 24-bit RGB Color\n
		bmpmono: Black-and-White Color\n
		bmpgray: Grayscale Color\n
		bmpsep1:\n
		bmpsep8:\n
		bmp16: 4-bit Color\n
		bmp256: 8-bit Color\n
		bmp32b: 32-bit RGBA Color
	:type format: unicode
	:param dpi: DPI.
		기본값은 200DPI입니다.
		양의 정수만을 취합니다.
	:type dpi: int
	:raise Exception: pdf 변환에 문제가 발생했을때.
	:return: BMP 이미지 바이너리 데이터
	:rtype: bytes
	"""
	cmd = _ghostscriptCommand[:]
	cmd.insert(-2, '-sDEVICE={format}'.format(format=format)) #출력 형식
	cmd.insert(-2, '-r{dpi}x{dpi}'.format(dpi=dpi))  #DPI
	devnull = open(os.devnull, 'w')
	gs = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=devnull)
	bmp = gs.communicate(pdf)[0]
	devnull.close()
	if gs.returncode != 0:
		raise Exception('pdf -> bmp 변환에 문제가 있습니다.')
	return bmp
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
from trashcli.trash import TrashPutCmd
import io
import chardet
import HTMLParser
import re

__all__ = ['trashPut', 'cleanSubtitle']

def cleanSubtitle(content, type=None):
	"""
	자막 파일을 깨끗히 만듭니다.
	
	다음 문제를 해결해줍니다:
	
		* 자막 깨짐 현상 해결(대신 mplayer의 기본 인코딩은 utf8이여야 합니다).
		* 자막에 존재하는 html 이스케이프된 문자열을 정상적으로 출력되도록 수정합니다.
		* smi 파일에 sami태그가 빠진 경우 추가시켜 줍니다.
	
	.. note:: 오직, smi, ass 파일만 지원합니다.
	.. caution:: 변환 결과가 잘못 될 수도 있습니다.
	
	:param content: 자막 내용.
	:type content: unicode, bytes
	:param type: 자막 종류.\n
		u'smi': 주어진 내용은 smi 파일.
		u'ass':주어진 내용은 ass 파일.
		None: smi, ass파일에 대한 공통 작업만 수행.
	:return: 변환된 자막 내용을 유니코드 문자열 타입으로 반환합니다.
	:rtype: unicode
	"""
	if not isinstance(content, unicode):
		encoding = chardet.detect(content)['encoding']
		content = content.decode(encoding, errors='replace')
	
	#smi 파일에 대한 처리.
	if type == 'smi':
		#sami 태그가 존재하지 않을 시 추가
		if not re.match(r'<SAMI>', content):
			content = '<SAMI>' + content
	
	#&#[0-9]+; 이스케이프 해제
	unescape = HTMLParser.HTMLParser().unescape
	buffer = lambda m: unescape(m.group(0))
	content = re.sub(r'&#\d+;', buffer, content)
	
	return content

def trashPut(target):
	"""
	대상을 휴지통에 버립니다.
	
	trashcli 패키지의 trash-put 명령어에 대한 인터페이스.
	
	:param taget: 지울 대상의 경로 문자열
	:type taget: unicode
	
	:return: True. 성공시
	:return: False. 문제가 존재 할 시
	:rtype: bool
	"""
	out = io.BytesIO()
	err = io.BytesIO()
	TrashPutCmd(out, err).run(['', '--', target.encode('utf8')])
	if err.getvalue():
		return False
	return True



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

def unpadPkcs5(data):
	"""
	PKCS#5 표준 패딩을 제거한다.
	
	:param data: 입력 데이터
	:type data: bytes
	:returns: 패딩이 제거된 데이터
	:rtype: bytes
	"""
	return data[0:-ord(data[-1])]
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import __builtin__ as builtins
import ANSIColors

def print_(*objects, **kwargs):
	"""
	디버깅 메시지를 터미널에 출력합니다.
	
	빨간색으로 강조된 '[디버그]' 문자열 뒤에 주어진 objs가 출력됩니다.
	
	.. @link http://pythonhosted.org/ANSIColors-balises/ANSIColors.html ANSIColors Module — ANSIColors-balises public documentation
	
	:param objects: 출력할 오브젝트들
	:type objects: object
	:param kwargs: 옵션(__builtin__.print 함수의 인자와 같습니다)
	"""
	buffer = ANSIColors.sprint('<Red>[DEBUG]<reset> ')
	builtins.print(buffer, end='');
	builtins.print(*objects, **kwargs);
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import sys

# terminal module
from .terminal import color
sys.modules['ufp.terminal.color'] = color
from .. import terminal
terminal.color = color

# image module
from . import image as s
from .. import image as d
d.RGB_MIN_VALUE = s.RGB_MIN_VALUE
d.RGB_MAX_VALUE = s.RGB_MAX_VALUE
d.GRAYSCALE_MIN_VALUE = s.GRAYSCALE_MIN_VALUE
d.GRAYSCALE_MAX_VALUE = s.GRAYSCALE_MAX_VALUE



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
from PyQt4.QtCore import pyqtSlot, QObject
import subprocess
import os

class PulseProgress(QObject):
	"""
	진행 표시줄이 끝없이 움직이는 진행 표시창을 띄웁니다.
	
	사용 예제는 다음과 같습니다.
		
	.. code-block:: python

		>>> import ufp.gui
		>>> import time
		>>> a = ufp.gui.PulseProgress('title', 'message')
		>>> a.open(); time.sleep(3); a.close()
		
	"""
	def __init__(self, title, message):
		"""
		진행 표시창을 초기화합니다.
		
		:param title: 제목
		:type title: unicode
		:param message: 메시지
		:type message: unicode
		"""
		super(PulseProgress, self).__init__()
		self.title = title
		self.message = message
		pass
	
	@pyqtSlot()
	def open(self):
		"""
		진행 표시창을 엽니다.
		"""
		self._devnull = open(os.devnull, 'w')
		self._zenity = subprocess.Popen(['zenity', '--progress', '--text', self.message, '--pulsate', '--no-cancel', '--auto-close', '--title', self.title], stdin=subprocess.PIPE, stdout=self._devnull, stderr=self._devnull)
		self._zenity.stdin.write('y')
		pass
	
	@pyqtSlot()
	def close(self):
		"""
		진행 표시창을 닫습니다.
		"""
		self._zenity.terminate()
		self._devnull.close()
		pass
	


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import sys
import ANSIColors

__all__ = ['feed', 'pause']

def feed(*objects):
	"""
	기존 줄을 지우고, 입력한 메시지를 화면에 씁니다.
	
	:param objects: 화면에 출력할 오브젝트
	:type objects: object
	"""
	ANSIColors.clearLine()
	print(*objects, end='')
	sys.stdout.flush()
	pass

def pause():
	"""
	사용자의 입력을 대기합니다.
	
	엔터를 눌러야 입력 대기가 끝납니다.
	"""
	print('[Enter]를 눌러 다음으로 진행합니다...', end = '');
	raw_input("");
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals
import tempfile
import urllib
import re
import cookielib
import chardet

from . import path as _p_path
from . import string as _p_string

def trimFilename(filename, **options):
	"""
	웹에서 다운받은 파일의 이름을 손질함.
	
	다음 작업을 수행합니다:
	
		- url coding 풀기
		- 사용불가능한 문자를 대체문자로 치환
		- 웹에서의 공백치환을 감지하고 경우에 따라 해제
		- 파일 이름을 다듬기
		- UHC로 변환가능한 인코딩은 변환
	
	.. caution:: 인코딩 변환이 잘못될수도 있습니다.
	
	:param filename: 파일명 유니코드 문자열
	:type filename: unicode, bytes
	:param from_encoding: 입력 인코딩.\n
		'auto' : 자동으로 인코딩을 파악합니다. \n
		False : 인코딩을 변환하지 않습니다. (기본)\n
		기타('utf8', 'uhc', ...)
	:param consider_extension: 확장자 고려 여부\n
		True: 확장자를 고려하여 작업합니다.\n
		False: 확장자를 고려하지 않습니다. (기본)
	:type consider_extension: bool
		
	:return: 변환된 문자열
	:return: u'Unknown'. 결과물이 공백이 될 경우 공백 대신 이 문자열이 반환됩니다.
	:rtype: unicode
	"""
	#옵션 초기값 설정
	options.setdefault(u'consider_extension', False)
	options.setdefault(u'from_encoding', False)
		
	#옵션 처리 : consider_extension
	considerExtension = options[u'consider_extension']
	
	#옵션 처리 : from_encoding
	if options[u'from_encoding'] == u'auto':
		fromEncoding = chardet.detect(filename)['encoding']
	elif options[u'from_encoding'] == False:
		fromEncoding = None
	else:
		fromEncoding = options[u'from_encoding']
	
	#url 디코딩
	filename = urllib.unquote(filename)
	
	#인코딩 변환
	if fromEncoding:
		filename = filename.decode(fromEncoding, errors='replace')
	
	#url 디코딩
	filename = urllib.unquote(filename)
	
	#전체 파일명의 앞뒤 공백 제거
	filename = filename.strip()
	
	#파일명에 사용불가능한 문자 치환
	filename = _p_path.replaceSpiecalChar(filename)
	
	#확장자 분리
	if considerExtension:
		ext = _p_path.extension(filename)
		if ext:
			filename = _p_path.filename(filename)
		else:
			considerExtension = False
		
	#이름에 포함된 and표시 제거
	if not re.search(u' ', filename):
		pattern1 = re.compile(u'[_+]', re.UNICODE)
		pattern2 = re.compile(ur'\.', re.UNICODE)
		if pattern1.search(filename):
			filename = pattern1.sub(' ', filename)
		elif pattern2.search(filename):
			filename = pattern2.sub(' ', filename)
	
	#파일이름의 앞뒤 공백 제거
	filename = filename.strip()
	
	#제어문자 제거
	filename = _p_string.removeControlChar(filename)
	#filename = re.sub('[:cntrl:]', ' ', filename, flags=re.UNICODE)
	
	#파일명 합침.
	if considerExtension:
		filename = '.'.join((filename, ext))
	
	#파일명을 반환
	if filename:
		return filename
	return 'Unknown'
	
def dequoteJsStr(jsStr) :
	"""
	자바 스크립트를 위해 콰우팅된 문자열을 콰우팅 해제시킵니다.
	
	:param jsStr: 콰우팅된 자바 스크립트 문자열
	:type jsStr: unicode
	:return: 디콰우팅 된 문자열.\n
		ex) abc\'asd\' -> abc'asd'
	:rtype: unicode
	"""
	REGEXS = [
		(r'\\', '\\'),
		(r"\'", "'"),
		(r'\"', '"'),
		(r'\n', '\n')
	];
	for before, after in REGEXS :
		jsStr = jsStr.replace(before, after)
	return jsStr;

def loadNetscapeCookie(session, cookiePath):
	"""
	Netscape 타입의 쿠키를 가져와서 requests session에 설정합니다.
	
	.. @remark 쿠키 헤더\n
	  # Netscape HTTP Cookie File\n
	  # http://www.netscape.com/newsref/std/cookie_spec.html\n
	  # This is a generated file!  Do not edit.
	
	:param session: requests 세션 객체
	:type session: requests.sessions.Session
	:param cookiePath: 쿠키 파일 경로 문자열
	:type cookiePath: unicode
	"""
	#임시 파일 생성
	with tempfile.NamedTemporaryFile('wb', prefix='.tmp_', suffix='.cookie') as tmpCookie:
		#넷스케이프 헤더 작성
		tmpCookie.write('# Netscape HTTP Cookie File\n')
		tmpCookie.write('# http://www.netscape.com/newsref/std/cookie_spec.html\n')
		tmpCookie.write('# This is a generated file!  Do not edit.\n')
		tmpCookie.write('\n')
		
		#기존 쿠키 파일의 내용을 삽입; 윈도우식 줄바꿈 -> 리눅스식으로 치환.
		with open(cookiePath, 'r') as f:
			buffer = f.read().replace('\r', '\n')
			tmpCookie.write(buffer.encode('UTF-8'))
		tmpCookie.flush()
		
		#쿠키 파일 불러오기
		cookieJar = cookielib.MozillaCookieJar(tmpCookie.name)
		cookieJar.load(ignore_discard=True, ignore_expires=True)
		session.cookies = cookieJar
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

def chunks(list, step):
	"""
	list를 step 단위로 묶어 yield합니다.
	
	.. 아래 함수가 정상적으로 작동 이유. 파이썬에서는 범위에서 벗어난 인덱스에 접근하려 할때, 자동으로 마지막 요소까지만 접근하도록 조정해준다.
	.. 참조 : http://stackoverflow.com/questions/312443/how-do-you-split-a-list-into-evenly-sized-chunks-in-python?page=1&tab=votes#tab-top
	
	예를 들어, 다음과 같이 작동합니다.
	
	.. code-block:: python
	
		>>> import ufp.list
		>>> for i in ufp.list.chunks(range(20), 5):
		... 	print(i)
		...
		[0, 1, 2, 3, 4]
		[5, 6, 7, 8, 9]
		[10, 11, 12, 13, 14]
		[15, 16, 17, 18, 19]
	
	:param list: 리스트
	:type list: list
	:param step: 단위
	:type step: int
	:yield: 각 step단위로 묶인 리스트. [...]
	"""
	length = len(list)
	for index in xrange(0, length, step):
		yield list[index:index+step]
	pass



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import unicode_literals, absolute_import

__title__ = 'ufp'
__version__ = '1.7.1'
__author__ = '별님'
__author_email__ = 'w7dn1ng75r@gmail.com'
__license__ = 'GPL v3'
__copyright__ = 'Copyright 2015 별님'
__url__ = 'https://github.com/Thestars3/pyufp/'

from .ufp import *

#하위 호환성 유지를 위한 설정
from . import _compatibleness



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import UserDict

class Lazy(UserDict.UserDict):
	"""
	설정된 사전 값에 대해 접근 할때 초기화되도록 설정 할 수 있는 사전.
	
	늦은 초기화가 생략 된 경우, 해당 키의 값에 대해서는 늦은 초기화를 사용하지 않도록 설정되어 있습니다.
	
	사용 예는 다음과 같습니다.
	
	.. code-block:: python
		
		>>> import ufp.dict
		>>> from ufp.terminal.debug import print_ as debug
		>>> dict = ufp.dict.Lazy()
		>>> def function():
		... 	debug('inited...')
		... 	return 10
		... 
		>>> dict.add('lazyInitValue', function, True)
		>>> dict['lazyInitValue']
		[DEBUG] inited...
		10
		>>> dict['lazyInitValue']
		10
	"""
	def __init__(self, dict=None):
		self.data = {}
		if dict is not None:
			self.update(dict)
		self._lazyInitDict = {}
	
	def add(self, key, value, lazyInit=False):
		"""
		늦은 초기화 여부를 설정하면서 사전에 등록합니다.
		
		:param key: 키
		:param value: 값
		:param lazyInit: 늦은 초기화 설정. True시 늦은 초기화를 사용함.
		"""
		self[key] = value
		self._lazyInitDict[key] = lazyInit
	
	def items(self):
		"""
		기존 UserDict.UserDict.items와 같습니다.
		"""
		for key, value in self.data.items():
			if self._lazyInitDict.get(key, False) and callable(value):
				value = value()
				self.data[key] = value
				self._lazyInitDict[key] = False
			yield key, value
	
	def values(self):
		"""
		기존 UserDict.UserDict.values와 같습니다.
		"""
		return map(lambda k,v: v, self.items())
	
	def __delitem__(self, key):
		if key in self._lazyInitDict:
			del self._lazyInitDict[key]
		del self.data[key]
	
	def setLazyInit(self, key, lazyInit):
		"""
		개별 키에 대한 늦은 초기화 여부를 설정합니다.
		
		:param key: 키
		:param lazyInit: 늦은 초기화 설정. True시 늦은 초기화를 사용함.
		"""
		self._lazyInitDict[key] = lazyInit
	
	def __getitem__(self, key):
		if self._lazyInitDict.get(key, False) and callable(self.data[key]):
			self.data[key] = self.data[key]()
			self._lazyInitDict[key] = False
		return self.data[key]


def setdefault(dict, key, default, empty=None):
	"""
	주어진 dict에 key가 존재하지 않으면, default를 설정합니다. 이때, key가 존재하더라도 empty와 같다면 default를 설정합니다. 
	
	사전 객체의 모든 요소와 empty값은 __eq__ 메소드가 작성되어 있어야 합니다. None은 예외.
	
	:param dict: 사전
	:type dict: dict
	:param key: 키
	:param default: 기본값
	:param empty: 이미 설정된 값이 empty와 같다면 default값을 설정함.
	"""
	if key not in dict:
		dict[key] = default
		return
	
	if dict[key] is None or dict[key] == empty:
		dict[key] = default
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess
import tempfile
import pattern.web
import tidylib

def clean(html, inputEncoding = "utf8") :
	"""
	html 문서를 보다 규격화된 xhtml로 변환합니다.
	
	:param html: html 내용
	:type html: unicode, str
	:param inputEncoding: 입력 문자열의 인코딩
	:type inputEncoding: unicode
	:returns: xhtml 문서
	:rtype: unicode
	.. 
	  @link http://tidy.sourceforge.net/docs/quickref.html HTML Tidy 설정 옵션 빠른 참조
	  @link http://countergram.com/open-source/pytidylib/docs/index.html#small-example-of-use Html Tidy에 관한 파이썬 인터페이스 pytidylib.moudule
	"""
	options = {
		str("output-xhtml"): True, #"output-xml" : True
		str("quiet"): True,
		str("show-errors"): 0,
		str("force-output"): True,
		str("numeric-entities"): True,
		str("show-warnings"): False,
		str("input-encoding"): inputEncoding,
		str("output-encoding"): "utf8",
		str("indent"): False,
		str("tidy-mark"): False,
		str("wrap"): 0
		};
	document, errors = tidylib.tidy_document(html, options=options)
	return document

def toText(html, converter='pattern.web', linebreaks=10, strip=False) :
	"""
	html 문서를 텍스트 문서로 변환합니다.
	
	이 함수는 다음과 같이 사용합니다.
		
	.. code-block:: python

		>>> import ufp.html
		>>> import requests
		>>> html = requests.get('http://www.gnu.org/licenses/').content
		>>> ufp.html.toText(html.decode('utf8'))
		u"Licenses\\n- GNU Project - Free Software Foundation\\n\\n\\n ...

	:param html: 원본 html 텍스트
	:type html: unicode
	:param converter: 변환에 사용할 변환기\n
		w3m : w3m 외부 프로그램을 불러와 작업을 하기 때문에 속도가 상당히 느립니다.\n
		pattern.web : pattern(http://www.clips.ua.ac.be/pattern) 라이브러리를 사용합니다.
	:type converter: unicode
	:param linebreaks: 줄바꿈 문자가 이어질 최대 라인 수. '\\\\n'가 linebreaks이상 연속되지 않도록 합니다. 그 이상의 '\\\\n'은 자동으로 제거됩니다. 만약 None으로 설정될 경우 이 옵션은 비활성됩니다. None 또는 1 이상의 값이어야 합니다.
	:type linebreaks: int, None
	:raise ValueError: 지원하지 않는 변환기를 입력 한 경우
	:return: text
	:rtype: unicode
	"""
	if converter == 'w3m':
		with tempfile.NamedTemporaryFile('wb', prefix='.tmp_', suffix='.html') as tempFile:
			tempFile.write(html.encode('UTF-8'))
			tempFile.flush()
			text = subprocess.check_output(['w3m', '-cols', '98304', '-dump', tempFile.name])
		if linebreaks is not None:
			text = pattern.web.collapse_linebreaks(text, linebreaks)
		if strip:
			text = text.strip()
		return text.deocde('UTF-8')
	
	if converter == 'pattern.web':
		html = pattern.web.strip_javascript(html)
		html = pattern.web.strip_inline_css(html)
		html = pattern.web.strip_forms(html)
		html = pattern.web.strip_comments(html)
		html = html.replace("\r", "\n")
		html = pattern.web.strip_tags(html, exclude=[], replace=pattern.web.blocks)
		html = pattern.web.decode_entities(html)
		html = pattern.web.collapse_spaces(html, True)
		text = pattern.web.collapse_tabs(html, True)
		if linebreaks is not None:
			text = pattern.web.collapse_linebreaks(text, linebreaks)
		if strip:
			text = text.strip()
		return text
	
	raise ValueError("'{converter}'는 지원하지 않는 변환기입니다.".format(converter=converter))



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import subprocess
import re
import os

def pgrep(pattern, extraArgs=list()):
	"""
	pattern에 해당하는 이름을 가진 프로세스 목록을 반환합니다.
	
	:param pattern: 프로세서 이름의 패턴
	:type pattern: unicode
	:param extraArgs: 추가 옵션
	:type extraArgs: list(unicode, ...)
	:return: 만약 pgrep 프로세서가 오류가 존재함을 알린(0과 1이 아닌 종료코드를 반환)다면 그 종료 코드를 반환합니다.
	:return: 만약 찾지 못한다면 빈 list를 반환합니다.
	:return: 찾는다면, [(id, name), ...]의 리스트를 반환합니다.
	"""
	args = ['pgrep', '-l', '--', pattern] + extraArgs
	with open(os.devnull, 'w') as devnull:
		ps = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=devnull)
		ps.wait()
		if ps.returncode == 0:
			buffer = ps.stdout.read()
			return re.findall('^(\d+)\s+(.*)$', buffer, re.MULTILINE)
		elif ps.returncode == 1:
			return list()
		else: 
			return ps.returncode
	pass

def quote(string):
	"""
	shell에서 주어진 문자열이 해석되지 않도록 콰우팅합니다.
	
	:param string: 문자열
	:type string: unicode
	:return: 콰우팅 된 문자열
	:rtype: unicode
	"""
	buffer = string.replace("'", r"'\''")
	return "'{string}'".format(string=buffer)
	


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import os
import re
import urllib
import urlparse
import os.path
import datetime
import shutil

def remove(path, force=False):
	"""
	주어진 경로에 존재하는 대상을 삭제합니다.
	
	폴더의 경우 재귀적으로 폴더 내용을 삭제한뒤 폴더를 삭제합니다.
	
	:param path: 삭제 대상의 경로
	:type path: unicode
	:param force: 오류를 무시할지 여부.\n
		True : 오류를 무시함.\n
		False : Exception이 발생하면, 그대로 던짐.
	:type force: bool
	"""
	try:
		if os.path.isdir(path):
			shutil.rmtree(path, force)
		else:
			os.remove(path)
	except Exception, e:
		if force:
			return
		raise e
	pass

def walk(top, maxDepth=None, onerror=None, followlinks=False):
	"""
	재귀적으로 경로 탐색하기
	
	'.'를 인자로 주었다면, './a', './b' 순으로 점점 더 깊게 탐색합니다.
	
	이 함수는 다음과 같은 식으로 사용한다.
		
	.. code-block:: python

		>>> import ufp.path
		>>> for root, dirs, files in ufp.path.walk('.', maxDepth=0):
		...		print(root, dirs, files)
		...		
		'.', ['ufp'], []
		
	:param top: 탐색 대상 경로
	:type top: unicode
	:param onerror: 오류가 발생하면, 주어진 주소의 함수를 호출하여 오류 인자를 넘겨줍니다.
	:type onerror: function
	:param followlinks: 심볼릭 링크된 경로를 포함하여 탐색합니다.
	:type followlinks: bool
	:param maxDepth: 탐색 할 최대 깊이.
		0이상의 정수여야 함.\n
		ex) 0 : 지정된 폴더만.
	:type maxDepth: int
	:yield: 현재 탐색 중인 위치(unicode), 폴더 이름([unicode, ...]), 파일 이름([unicode, ...])
	"""
	top = top.rstrip(os.path.sep)
	assert os.path.isdir(top)
	num_sep = top.count(os.path.sep)
	for root, dirs, files in os.walk(top, onerror=onerror, followlinks=followlinks):
		yield root, dirs, files
		if maxDepth is not None and num_sep + maxDepth <= root.count(os.path.sep):
			del dirs[:]
	pass
	
def moveAllContent(dirPath, destPath):
	"""
	dirPath에 존재하는 모든 파일을 destPath로 옮깁니다.
	
	만약 중복 파일이 존재한다면 자동으로 중복 회피한 이름을 새 이름으로 지정하여 옮깁니다. 중복 회피 대상은 각 디렉토리 바로 아래의 파일들입니다.
	
	:param dirPath: 이동 할 대상 파일들이 담긴 폴더
	:type dirPath: unicode
	:param destPath: 파일들이 옮겨져갈 위치
	:type destPath: unicode
	"""
	for src in listdir(dirPath, fullpath=True):
		#이동 경로 설정
		buffer = os.path.basename(src)
		buffer = os.path.join(destPath, buffer)
		dest = unique(buffer)
		
		shutil.move(src, dest) #이동
	pass

def mergeDir(dirPaths, destPath):
	"""
	dirPaths에 존재하는 모든 파일을 destPath로 옮긴 뒤, dirPath를 삭제합니다.
	
	만약 중복 파일이 존재한다면 자동으로 중복 회피한 이름을 새 이름으로 지정하여 옮깁니다. 중복 회피 대상은 각 디렉토리 바로 아래의 파일들입니다.
	
	:param dirPaths: 이동 할 대상 파일들이 담긴 디렉토리 경로, 또는 디렉토리 경로 목록.
	:type dirPaths: unicode, list(unicode, ...), tuple(unicode, ...)
	:param destPath: 파일들이 옮겨져갈 위치
	:type destPath: unicode
	"""
	if isinstance(dirPaths, unicode):
		dirPaths = [dirPaths]
	
	for dirPath in dirPaths:
		moveAllContent(dirPath, destPath)
		os.rmdir(dirPath)
	pass

def mtime(path, format):
	"""
	path에 해당하는 파일의 최근 수정 시각을 지정된 format형식으로 작성하여 반환합니다.
	
	:param path: 경로 문자열
	:type path: unicode
	:param format: 형식 문자열(datetime strftime에서 지원하는 형식)
	:type format: unicode
	:return: path에 존재하는 파일의 최근 수정 시각의 format에 지정된 형식
	:rtype: unicode
	"""
	buffer = os.path.getmtime(path)
	buffer = datetime.fromtimestamp(buffer).strftime(format)
	return buffer
	
def pjoin(parentPath, filenames):
	"""
	parentPath를 각 filenames의 아이템과 결합합니다.
	
	:param parentPath: 부모 경로
	:type parentPath: unicode
	:param filenames: 파일명 리스트.
	:type filenames: list(unicode, ...)
	:return: 결합된 리스트.
		[parentPath/filename, parentPath/filename, ...]
	:rtype: list(unicode, ...)
	"""
	result = list()
	for filename in filenames:
		buffer = os.path.join(parentPath, filename)
		result.append(buffer)
		pass
	return result

def listdir(path, **options):
	"""
	지정된 폴더에서 pattern에 해당하는 파일 및 폴더의 목록을 가져옵니다.
	
	:param path: 목록을 가져올 폴더 경로
	:type path: unicode
	:param pattern: 필터 패턴(대소문자를 구분하지 않습니다)\n
		string : 패턴 문자열\n
		[string, ...]: 패턴 문자열 리스트\n
		None: 필터링 하지 않음
	:param patternReverse: \n
		True : 필터에 매칭되지 않는 대상만 얻습니다.\n
		False : 필터에 매칭되는 대상만 얻습니다.
	:type patternReverse: bool
	:param fullpath: 반환시 주어진 경로를 포함한 경로를 반환합니다.\n
		True : [path + filename, ...]\n
		False : [filename, ...]
	:type fullpath: bool
	:param sortKey: 정렬키 함수\n
		해당 함수에는 전체 경로가 전달 됩니다.
	:type sortKey: function
	
	:return: 필터링된 폴더 내용
	:rtype: list(unicode, ...)
	"""
	options.setdefault('pattern', None)
	options.setdefault('patternReverse', False)
	options.setdefault('fullpath', False)
	options.setdefault('sortKey', None)
	
	if options['pattern'] is not None:
		pattern = options['pattern']
		
		#만약 패턴이 문자열이라면 리스트에 담기.
		if isinstance(pattern, unicode):
			pattern = list(pattern)
		
		#패턴 컴파일
		pattern = map(lambda x: re.compile(x, re.IGNORECASE), pattern)
	
	#목록 작성
	listdir = list()
	iterator = os.listdir(path)
	#패턴 옵션이 설정된 경우 필터링
	if options['pattern'] is not None:
		for filename in iterator:
			for p in pattern:
				#패턴 역전 옵션이 꺼진 경우
				if p.search(filename) and not options['patternReverse']:
					listdir.append(filename)
					break
				
				#패턴 역전 옵션이 켜진 경우
				if options['patternReverse']:
					listdir.append(filename)
					break
	else:
		listdir.extend(iterator)
	
	#sortKey 옵션이 사용된 경우
	if options['sortKey']:
		key = options['sortKey']
		buffer = lambda x: key(os.path.join(path, x))
		listdir.sort(key=buffer)
	
	#fullpath 옵션이 켜진 경우
	if options['fullpath']:
		listdir = pjoin(path, listdir)
	
	#결과 반환
	return listdir
	
def toUrl(path):
	"""
	path를 file:///형식으로된 주소로 되돌려줍니다.
	
	:param path: 경로 문자열.\n
		ex) ../home/ufp
	:type path: unicode
	:return: file:///형식으로된 주소
	:rtype: unicode
	"""
	buffer = urllib.pathname2url(path.encode('UTF-8'))
	return urlparse.urljoin('file:', buffer)

def replaceSpiecalChar(string, **options) :
	"""
	운영체제의 경로에서 사용 불가능한 문자열을 대체문자로 치환합니다.
	
	윈도우 및 유닉스 계열 운영체제에서 파일에 포함되면 문제가 되는 특수문자를 대체문자로 치환합니다. 경로 구분자를 대체 문자로 치환합니다.
	
	.. note::
		윈도우에서 파일명으로 사용 할 수 없는 문자들(파일 이름에 다음 문자를 사용할 수 없습니다):\n
		\\\\ / : * ? " < > |
	.. note::
		리눅스에서 파일명으로 사용 할 수 없는 문자들(파일 이름에 다음 문자를 사용할 수 없습니다):\n
		/
	
	:param string: 치환 될 문자열
	:tpye string: unicode
	:param **options: \n
		type: u'windows', u'unix'.
		윈도우의 경로 구분 문자는 리눅스에서 표시되는 모양이 다릅니다. 지정된 OS에서 보여지는 문자 모양과 유사한 대체문자로 치환하도록 지정합니다. 기본값은 unix입니다.\n
		keep_path_characters(bool): 경로 구분자를 치환하지 않습니다. 이 설정은 type 설정에 의존합니다. 기본 값은 False입니다.
	:rtype: 
	"""
	UNIX_PATH_CHARACTER_RE = ("/", "／");
	ESCAPE_CHARTER_UNIX_TYPE_RE = ("\\", "＼")
	WINDOWS_PATH_CHARACTER_RE = ("\\", "￦")
	DEFAULT_REGEXS = [
		("?", "？"),
		(":", "："),
		("*", "＊"),
		('"', "＂"),
		("<", "〈"),
		(">", "〉"),
		("|", "│"),
		("'", "＇"),
		("$", "＄"),
		("!", "！")
	];
	
	#옵션 초기값 설정
	options.setdefault('type', 'unix')
	options.setdefault('keep_path_characters', False)
	
	#옵션 처리
	regexs = DEFAULT_REGEXS
	if options['type'] == 'unix' :
		regexs.append(ESCAPE_CHARTER_UNIX_TYPE_RE);
		if not options['keep_path_characters'] :
			regexs += [UNIX_PATH_CHARACTER_RE];
	elif options['type'] == 'windows' :
		regexs += [UNIX_PATH_CHARACTER_RE];
		if not options['keep_path_characters'] :
			regexs += [WINDOWS_PATH_CHARACTER_RE];
	
	for before, after in regexs :
		string = string.replace(before, after);
	
	return string;

def dirname(path) :
	"""
	주어진 경로의 부모 경로를 추출해냅니다.
	
	만약 'abc'가 주어졌다면, 반환값은 '.'입니다.
	
	.. note:: os.path.dirname('asd') -> ''이 되는 문제를 해결하기 위해 만들어졌습니다.
	
	:param path: 경로 문자열.
	:type path: unicode
	:return: 부모 경로
	:rtype: unicode
	"""
	dirnameRe = re.compile(r'(?P<dirname>^.*)/', re.DOTALL | re.UNICODE).search(path);
	if dirnameRe :
		return dirnameRe.group('dirname');
	return '.';

def unique(targetPath, spliteExt = True) :
	"""
	해당 경로에서 유일한 경로를 만들어 냅니다.
	
	대소문자를 구분하지 않습니다. 만약 'a/b/c'를 인자로 준다면, `a/b/c d(1)`식으로 충복을 회피처리한 결과를 반환합니다.
	
	다음과 같은 식으로 사용합니다.
	
	.. code-block:: python
	
		>>> import ufp.path
		>>> import os
		>>> os.listdir('.')
		['test.file']
		>>> ufp.path.unique('./test.file')
		./test (d1).file
		>>> f.open('./test (d1).file', 'w').close()
		>>> ufp.path.unique('./test.file')
		./test (d2).file
	
	:param targetPath: 주어지는 값은 유니코드 문자열이여야 함.
	:type targetPath: unicode
	:return: 만약, 주어진 경로가 충돌하지 않는다면 주어진 경로를 그대로 반환합니다.
	:return: 만약 주어진 경로의 부모 경로가 존재하지 않는다면, 주어진 문자열 그대로 반환합니다.
	:return: 유일한 경로.
	"""
	#경로 분할
	targetDirname = dirname(targetPath)
	targetBasename = os.path.basename(targetPath)
	
	#부모 경로가 존재하는지 확인
	if not os.path.exists(targetDirname):
		return targetPath
	
	#해당 경로의 목록을 작성
	fileList = ['.', '..']
	for dirpath, dirnames, filenames in os.walk(targetDirname):
		fileList.extend(filenames)
		break
		
	#중복되는 대상이 존재하는지 확인
	existDuplicateFile = False;
	buffer = re.escape(targetBasename);
	fullmatchRe = re.compile(r"^{targetBasename}$".format(targetBasename=buffer), re.IGNORECASE | re.UNICODE)
	for fileName in fileList:
		if fullmatchRe.search(fileName):
			existDuplicateFile = True;
			break
		pass
	
	#중복되는 대상이 존재하고 있지 않다면 이름을 그대로 돌려줌.
	if not existDuplicateFile:
		return targetPath;
		
	#파일명과 확장자를 추출
	if spliteExt :
		targetFileExt = extension(targetBasename);
		if targetFileExt :
			targetFileName = filename(targetBasename);
		else :
			targetFileName = targetBasename;
			spliteExt = False;
	else :
		targetFileName = targetBasename;
	
	#중복 파일들의 숫자를 가져옴.
	escapedTargetFileName = re.escape(targetFileName);
	if spliteExt :
		extractDupCountRe = re.compile(r"^{escapedTargetFileName} \(d(?P<number>[0-9]+)\)\.{targetFileExt}$".format(escapedTargetFileName=escapedTargetFileName, targetFileExt=targetFileExt), re.DOTALL | re.IGNORECASE | re.UNICODE);
	else :
		extractDupCountRe = re.compile(r"^{escapedTargetFileName} \(d(?P<number>[0-9]+)\)$".format(escapedTargetFileName=escapedTargetFileName), re.DOTALL | re.IGNORECASE | re.UNICODE);
	counts = [];
	for fileName in fileList :
		m = extractDupCountRe.search(fileName);
		if m :
			buffer = m.group('number')
			buffer = int(buffer)
			counts.append(buffer);
		pass
	
	#중복 숫자를 설정
	if counts :
		counts.sort();
		notDuplicatedNumber = counts[-1] + 1;
	else :
		notDuplicatedNumber = 1;
		
	#중복 회피 이름 생성
	if spliteExt :
		uniqueName = "{targetFileName} (d{notDuplicatedNumber}).{targetFileExt}".format(targetFileName=targetFileName, notDuplicatedNumber=notDuplicatedNumber, targetFileExt=targetFileExt)
	else :
		uniqueName = "{targetFileName} (d{notDuplicatedNumber})".format(targetFileName=targetFileName, notDuplicatedNumber=notDuplicatedNumber)
	
	return os.path.join(targetDirname, uniqueName);

def filename(filePath) :
	"""
	파일 경로로 부터 확장자를 제외한 파일명을 추출해냅니다.
	
	:param filePath: 주어지는 값은 유니코드 문자열이여야 함.\n
		예를 들어, '../asd/.qwe.tar.bz2'가 인자로 주어진 다면 반환값은 '.qwe.tar' 입니다.
	:type filePath: unicode
	:return: 파일 이름
	:rtype: unicode
	"""
	rx = re.compile(r"^(.*/)?(?P<name_space>.+?)?(?P<ext_space>\.[a-z0-9]+)?$", re.DOTALL | re.IGNORECASE | re.UNICODE);
	
	result = rx.search(filePath)
	if not result:
		return unicode()
	
	nameSpace = result.group('name_space');
	if nameSpace :
		return nameSpace;
	
	extSpace = result.group('ext_space')
	if extSpace:
		return extSpace;
		
	return unicode()

def extension(fileName) :
	"""
	주어진 파일명의 확장자를 추출합니다.
	
	:param fileName: 주어지는 값은 유니코드 문자열이여야 함.
	:type fileName: unicode
	:return: '../asd/.qwe'가 인자로 주어진 다면 반환값은 (빈 값) 입니다.
	:return: 만약 확장자가 없다면, (빈 값)을 리턴합니다.
	:rtype: unicode
	"""
	extRe = re.compile(r"[^/]+\.(?P<ext>[a-z0-9]+)$", re.DOTALL | re.IGNORECASE | re.UNICODE);
	result = extRe.search(fileName);
	if result :
		return result.group('ext');
	return unicode();



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import PIL.Image
import PIL.ImageChops
import itertools
import math
import copy

RGB_COLOR_COUNT = 256
GRAYSCALE_COLOR_COUNT = 256 #PIL.Image의 L 모드를 의미.

def quantizeByImprovedGrayScale(image, colorCount=GRAYSCALE_COLOR_COUNT):
	"""
	주어진 이미지에 대해 Improved Gray Scale(IGS) 양자화시킵니다.
	
	이 함수는 원본을 변경시킵니다.
	
	다음과 같은 식으로 사용한다.
		
	.. code-block:: python

		>>> import ufp.image
		>>> import PIL.Image
		>>> im = PIL.Image.open('test.jpg', 'r')
		>>> im = im.convert('L')
		>>> ufp.image.quantizeByImprovedGrayScale(im).save('igs.jpg')
		
	..
	  @link http://stackoverflow.com/questions/14434963/change-number-of-gray-levels-in-a-grayscale-image-in-matlab change number of gray levels in a grayscale image in matlab - Stack Overflow
	  @link http://code.activestate.com/recipes/259112/ Improved Gray Scale Quantization (Python recipe)
	
	:raise ValueError: 사용 불가능한 모드의 이미지를 인자로 준 경우
	:param image: 이미지 객체. 이 이미지는 L 모드여야 합니다.
	:type image: PIL.Image
	:param colorCount: 원본의 색상 수.
		주어진 이미지의 색상 수를 목표로 하는 색상로 맞추는데 사용 할 수도 있습니다. 이 경우, 주어진 모드가 한 채널에서 가질 수 있는 최대 값을 상한선으로 하여, 목표로 하는 색상의 수를 정하십시오.
	:type colorCount: int
	:return: 변경된 이미지 객체
	:rtype: PIL.Image
	"""
	if image.mode != 'L':
		raise ValueError('L 모드의 이미지만 사용 할 수 없습니다.'.format(image.mode))
	
	raito = math.trunc(GRAYSCALE_COLOR_COUNT / colorCount)
	mask = raito - 1
	prev_sum = 0
	pixel = image.load()
	width, height = image.size
	for x in xrange(0, width):
		for y in xrange(0, height):
			value = pixel[x, y]
			if value/raito != mask:
				prev_sum = value + (prev_sum & mask)
			else:
				prev_sum = value
			pixel[x, y] = math.trunc(prev_sum/raito) * raito
			pass
		pass
	return image

def changeColorDepth(image, colorCount):
	"""
	이미지의 색 깊이를 감소시킵니다.
	
	이 함수는 원본을 변경시킵니다.
	
	..
	  @link http://en.wikipedia.org/wiki/List_of_monochrome_and_RGB_palettes List of monochrome and RGB palettes - Wikipedia, the free encyclopedia
	  @link http://stackoverflow.com/questions/14434963/change-number-of-gray-levels-in-a-grayscale-image-in-matlab change number of gray levels in a grayscale image in matlab - Stack Overflow
	
	:raise ValueError: 사용 불가능한 모드의 이미지를 인자로 준 경우
	:param image: PIL.Image 객체.
		L, RGB, RGBA 모드의 이미지를 처리 할 수 있습니다.
	:type image: PIL.Image
	:param colorCount: 결과물의 색상 수.
		주어진 모드가 한 채널에서 가질 수 있는 최대 값을 상한선으로 하여, 목표로 하는 색상의 수를 정하십시오.
	:type colorCount: int
	:return: 변경된 PIL.Image 이미지 객체
	:rtype: PIL.Image
	"""
	if image.mode == 'L':
		raito = GRAYSCALE_COLOR_COUNT / colorCount
		change = lambda value: math.trunc(value/raito)*raito
		return image.point(change)
	
	if image.mode == 'RGB' or image.mode == 'RGBA':
		raito = RGB_COLOR_COUNT / colorCount
		change = lambda value: math.trunc(value/raito)*raito
		return PIL.Image.eval(image, change)
	
	raise ValueError('{mode} 모드의 이미지는 사용 할 수 없습니다.'.format(mode=image.mode))

def mostPopularEdgeColor(image):
	"""
	이미지의 둘레(left,right,top,bottom)에서 가장 많이 나타나는 색깔이 무엇인지 계산한다.
	
	..
	  @link http://coreapython.hosting.paran.com/howto/sebsauvage_net-%20Snyppets%20-%20Python%20snippets.htm#autocrop 이 함수에 대해 참조한 자료가 있는 웹페이지
	
	:param image: PIL 이미지 객체
	:type image: PIL.Image
	:returns: 가장 많은 색상(정수 튜플 (R,G,B))
	:returns: L 모드 이미지인 경우 가장 많은 색상(정수)를 반환합니다.
	"""
	if image.mode not in ['RGB', 'L']:
		image = image.convert("RGB")
		pass
	
	# 이미지의 둘레에서 픽셀을 얻는다:
	width,height = image.size
	left   = image.crop((0, 1, 1, height-1))
	right  = image.crop((width-1, 1, width, height-1))
	top    = image.crop((0, 0, width, 1))
	bottom = image.crop((0, height-1, width, height))
	pixels = left.tostring() + right.tostring() + top.tostring() + bottom.tostring()
	
	if image.mode == 'RGB':
		# 어떤 RGB 삼중원소가 가장 많은지 계산한다
		counts = dict()
		for i in xrange(0, len(pixels), 3):
			rgb = pixels[i] + pixels[i+1] + pixels[i+2]
			if rgb in counts:
				counts[rgb] += 1
			else:
				counts[rgb] = 1
				pass
			pass
		
		# 가장 많은 색깔을 얻는다:        
		mostPopularColor = sorted([(count, rgba) for (rgba, count) in counts.items()], reverse=True)[0][1]
		r = ord(mostPopularColor[0])
		g = ord(mostPopularColor[1])
		b = ord(mostPopularColor[2])
		return (r, g, b)
	
	if image.mode == 'L':
		buffer = map(lambda i: ord(pixels[i]), xrange(0, len(pixels)))
		buffer = list(itertools.groupby(buffer))
		buffer.sort(key=lambda x: len(list(x[1])), reverse=True)
		return buffer[0][0]
	pass

def trim(image, backgroundColor=None, fuzz=0):
	"""
	이미지 주위에 있는 여백을 제거한다.
	
	이미지에 알파 채널이 있으면, 그것을 사용하여 무엇을 자를지 고른다. 그렇지 않으면, 이미지 둘레에서 가장 많은 색상을 찾으려고 시도한다. 그리고 이 색상을 여백으로 간주한다. (backgroundColor 매개변수로 이 색상을 오버라이드할 수 있다.)
	
	다음과 같은 식으로 사용한다.
	
	.. code-block:: python

		>>> import ufp.image
		>>> import PIL.Image
		>>> im = PIL.Image.open('test.jpg', 'r')
		>>> ufp.image.trim(im, fuzz=13.3).save('trim.jpg')
		
	.. warning:: fuzz옵션 사용시, 이미지가 여백과 유사한 색상일 경우 이미지의 경계를 넘어 이미지까지 잘리는 문제가 생길 수 있다.
	.. warning:: fuzz옵션 사용시, 여백에 잡음이 존재하는 경우 여백을 넘어 이미지를 자를 수 있습니다.
	
	:raise ValueError: fuzz값이 잘못된 경우
	:raise ValueError: 주어진 이미지가 처리 불가능한 모드인 경우
	
	..
	  @link http://coreapython.hosting.paran.com/howto/sebsauvage_net-%20Snyppets%20-%20Python%20snippets.htm#autocrop 이 함수에 대해 참조한 자료가 있는 웹페이지
	  @link http://stackoverflow.com/questions/10615901/trim-whitespace-using-pil 이 함수에 대해 참조한 자료가 있는 웹페이지
	  @link https://pillow.readthedocs.org/reference/ImageChops.html Pillow 사용자 문서 - ImageChops 모듈 API
	
	:param image: (PIL 이미지 객체) 다듬을 이미지.
		알파 채널이 포함된 이미지 또는, RGB, L 형식의 이미지만 처리 할 수 있습니다.
	:type image: PIL.Image
	:param backgroundColor: "다듬을 배경"으로 간주될 색상(RGB: 3개의 정수가 담긴 터플).
		예컨데, tuple(0,0,255)와 같은 형식으로 주어야 합니다. 이미지가 투명이면, 이 매개변수는 무시된다. 이미지가 투명이 아니고 이 매개변수가 주어지지 않으면, 자동으로 계산된다. 만약, L 모드라면 정수를 줘야 한다.
	:param fuzz: float 또는 int형(0~99.9). 배경색과 다른 색상을 동일하게 취급하는 정도를 설정합니다.
	:type fuzz: int, float
	
	:return: PIL.Image 객체.
	:return: 변경될 내용이 없는 경우, 원본을 반환합니다.
	:rtype: PIL.Image
	"""
	bbox = None
	
	#fuzz값 체크
	if not isinstance(fuzz, (float, int)) or not 0 <= fuzz or not fuzz <= 99.9:
		raise ValueError('fuzz에는 오직 0~99사이의 int 또는 float만 사용 할 수 있습니다.')
	
	#이미지에 투명 레이어가 있으면, 그걸 사용한다.
	if 'A' in image.getbands():
		#이는 투명 레이어가 있는 모든 모드에 작동한다
		buffer = image.getbands()
		buffer = list(buffer).index('A')
		buffer = image.split()[buffer]
		bbox = buffer.getbbox()
	#RGB 모드인 경우
	elif image.mode in ['RGB', 'L']:
		if not backgroundColor:
			backgroundColor = mostPopularEdgeColor(image)
			pass
		bg = PIL.Image.new(image.mode, image.size, backgroundColor) # 불투명 이미지를 다듬는다. .getbbox()는 언제나 검정색을 다듬는다. 그래서 이미지에서 "배경색"을 빼줄 필요가 있다.
		diff = PIL.ImageChops.difference(image, bg)  # 이미지에서 배경색을 뺀다.
		
		#차이를 감소시킨다.
		if fuzz > 0:
			if image.mode == 'L':
				max_ = GRAYSCALE_COLOR_COUNT
			else:
				max_ = RGB_COLOR_COUNT
			offset = - math.trunc(max_*fuzz/100)
			diff = PIL.ImageChops.add(diff, diff, 2.0, offset)
			pass
		
		bbox = diff.getbbox()  # 이미지의 진짜 둘레를 찾는다.
	else:
		raise ValueError('{mode} 모드의 이미지는 처리가 불가능 합니다.'.format(mode=image.mode))
	
	if bbox:
		image = image.crop(bbox)
		pass
	
	return image
